var _ = require("underscore");
var _text_ = require("./text");
var promiseSerial = require("./promises").promiseSerial;
var sizeof = require('object-sizeof');

var _emptyArray = [];

var chunk = function(array, chunkSize) {

    return array.reduce(function(p, cur, i) {
        (p[i/chunkSize|0] = p[i/chunkSize|0] || []).push(cur);
        return p;
        },[]);
};

var hasNoUndefinedItems = function (array) {

    var indexOfEmptyItem = _.findIndex(array, function (item) {

        return _.isUndefined(item);
    });
    
    return -1 == indexOfEmptyItem;
};

var pluckMany = function (collection, propertiesToPluck) {

    if(collection 
        && !_.isEmpty(collection)
        && hasNoUndefinedItems(collection)) {

        return _.map(collection, function (item) {

            var result = {};

            _.each(propertiesToPluck, function(property) {
                result[property] = item[property]; 
            });

            return result;
        });
    }
};

var processPartial = function (array, chunkSize, func) {

    return new Promise(function (resolve, reject) {

        if (_.isEmpty(array)) {

            return resolve();
        } else {

            if (isNaN(chunkSize)) {

                if (chunkSize.maxBytes) {

                    var sizeOfArrayInBytes = sizeof(array);
                    chunkSize = Math.floor(array.length / (sizeOfArrayInBytes / chunkSize.maxBytes));
                }
            } 

            if (isNaN(chunkSize)) {

                return reject({
                    message: "Invalid options",
                    data: {
                        chunkSize: chunkSize
                    }
                })
            }
            var chunks = chunk(array, chunkSize);

            var chunkPromises = chunks.map((arrayChunk, index) => () => {

                return func(arrayChunk, index);
            });

            promiseSerial(chunkPromises)
                .then(resolve)
                .catch(reject);
        }
    });
}

var isStringAnArray = function (str) {

    return null != matchIsArrayString(str);
};

function isNormalInteger(str) {

    return /^\+?(0|[1-9]\d*)$/.test(str);
};

function matchIsArrayString(str) {

    return null != str 
            && typeof str == "string" 
            && !isNormalInteger(str)
                ? str.match(/\[(.*?)\]/)
                : null;
};

function stringHasValuesWithDelimeter (value, delimiter) {

    return value ? 1 < value.split(delimiter).length : null;
};

function convertArrayWithStringsToInts (value, options) {

    if (value) {

        return _.map(value, function (resultValue) {

            var converted = resultValue;

            if (options && options.removeText) {

                converted = _text_.numbersOnly(converted);
            }

            var result;

            if(!isNaN(converted)) {

                result = parseInt(converted, 10);
            } else {

                result = converted.trim();
            }
            
            return result;
        });
    }

    return null;
};

var getArrayWithValue = function (value, options) {

    if(!value) return _emptyArray;

    var delimiter = (options && options.delimiter) ? options.delimiter : ",";

    if(_.isArray(value)) {
        
        return value;
    } else {

        if (_.isNumber(value)) {

            return [parseInt(value, 10)];
        } else if(isStringAnArray(value)) {

            if ('[]' == value) {
                return _emptyArray;
            }

            var match = matchIsArrayString(value);

            if (match) {

                var resultValues = match[1].split(delimiter);
                return convertArrayWithStringsToInts(resultValues, options);
            } else {

                return _emptyArray;
            }
            
        } else if (stringHasValuesWithDelimeter(value, delimiter)) {

            return convertArrayWithStringsToInts(value.split(delimiter), options);
        } else if (value) {

            return convertArrayWithStringsToInts([value], options);
        } else {

            return _emptyArray;
        }
    }
};

var prepareForAutocomplete = function (collection, titleField) {

    if (!collection || !titleField) {
        
        return null;
    }

    var plucked = pluckMany(collection, ["id", titleField]);
    var prepared = _.map(plucked, function (item) {
    
        return {
            id: item.id,
            text: item[titleField]
        };
    });

    return JSON.stringify(prepared);
};

var toString = function (array, delimiter) {

    if (!array || !_.isArray(array)) {

        return null;
    }

    delimiter = delimiter || ',';

    return array.join(delimiter);
};

/*
* [1, 2, 3] => [1, 2, 3]
* [1,2,3, [ { a: 1} ]] => [1,2,3, { a: 1} ]
*/
var flatten = function (array) {

    if(!array) {
        return null;
    }

    return [].concat.apply([], array);
};

module.exports = {
    
    emptyArray: _emptyArray,
    processPartial: processPartial,
    flatten: flatten,
    toString: toString,
    prepareForAutocomplete: prepareForAutocomplete,
    getArrayWithValue: getArrayWithValue,
    pluckMany: pluckMany,
    hasNoUndefinedItems: hasNoUndefinedItems
};