var imagemin = require('imagemin');
var imageminJpegtran = require('imagemin-jpegtran');
var imageminPngquant = require('imagemin-pngquant');
var Jimp = require("jimp");
var path = require('path');
var _ = require("underscore");
var deepExtend = require('deep-extend');
var _obLogger = require("./ob-logger")("hippodamus.images");
var _promiseSerial = require("./promises").promiseSerial;
var _file = require("./file");

function getResizedImagePath(fileName, suffix) {

   var re = new RegExp("(.+)\\.(jpg|jpeg|png|gif)", "g");
   return fileName.replace(re, "$1_" + suffix + ".$2");
}

function getImageFromFile(file, options) {

    return new Promise(function (resolve, reject) {

        optimizeImage(file, options).then(function () {

            try {

                //var image = _file.readFileInBase64Sync(file.path);
                var image = _file.readFileInBase64Sync(file.path);

                resolve(image);
            } catch (ex) {
                reject(ex)
            }
        }).catch(function (error) {

            _obLogger.error(error, "Can not optimize image");
            reject(error);
        });
    });
}

function getImagesFromFiles(files, options) {

    var imagePromises = files.map(file => () => {
        return getImageFromFile(file, options);
    });
    
    return _promiseSerial(imagePromises);
}

function buildResizedImages (files, sizes) {

    return new Promise(function (resolve, reject) {

        if(!sizes) {

            resolve([]);
        } else {

            var promises = [];

            _.each(sizes, function (size) {
                    
                var imagePromises = files.map(file => () => {
                    
                    return buildResizedImage(file, size);
                });

                Array.prototype.push.apply(promises, imagePromises);
            });

            _promiseSerial(promises)
                .then(function (images) {

                    resolve(images);
                })
                .catch(function (error) {

                    _obLogger.error(error, "Could not build resized images m8");
                    reject(error);
                });
        }
    });
}

function buildResizedImage (originalImage, imageSize) {

    return new Promise(function(resolve, reject) {
  
        if(!originalImage || !imageSize) {

            return reject();
        }

        Jimp.read(originalImage.path, function (err, image) {
            // do stuff with the image (if no exception)
    
            if (err) {
                
                _obLogger.error("Could not read " + originalImage.path);
                reject();
            } else if (image) {

                var w = image.bitmap.width; // the width of the image
                if (imageSize.width < w) {
                
                    image.resize(imageSize.width, Jimp.AUTO);
                }
    
                var resizedImagePath = getResizedImagePath(originalImage.path, imageSize.prefix);
                var resizedName = originalImage.name + "_" + imageSize.prefix;
                        
                image.write(resizedImagePath, function () {
                
                    resolve(deepExtend({}, originalImage, {
                        name: resizedName,
                        path: resizedImagePath,
                        size: imageSize,
                        prefix: imageSize.prefix
                    }));
                });
            }
        });
    });
}

function optimizeImage (file, options) {

    return new Promise(function (resolve, reject) {

        if (options && options.skipOptimization) {

            return resolve();
        }

        var pathParts = file.path.split(path.sep);
        var buildFileDirectory = pathParts.splice(0, pathParts.length - 1).join(path.sep);

        imagemin([file.path], buildFileDirectory, {
            plugins: [
                imageminJpegtran(),
                imageminPngquant({quality: '65-80'})
            ]
        }).then(function (files) {

            resolve(files);
        }, function (error) {

            _obLogger.error(error, "Can not optimize image");
            reject(error);
        });
    });
}

function takeRegularImages(images) {

    if(!images) {

        return null;
    }

    return _.filter(images, function (image) {

        var take = false;

        if (image.size) {

            take = image.size == IMAGESIZES.REGULAR;
        } else if (image.size_prefix) {

            take = image.size_prefix == IMAGESIZES.REGULAR.prefix;
        } else {

            //todo: to support old images and treat them as regular for now
            take = true;
        }

        return take;
    });
}

var IMAGESIZES = {

    REGULAR: {
        prefix: "regular"
    },

    SMALL: {
        prefix: "small",
        width: 150
    },
    
    MEDIUM: {
        prefix: "medium",
        width: 500
    },

    THUMBNAIL: {
        prefix: "thumbnail",
        width: 50
    }
};

module.exports = {

    IMAGESIZES: IMAGESIZES,

    takeRegularImages: takeRegularImages,
    
    getImageFromFile: getImageFromFile,
    getImagesFromFiles: getImagesFromFiles,
    getResizedImagePath: getResizedImagePath,

    buildResizedImages: buildResizedImages,
    buildResizedImage: buildResizedImage,
    optimizeImage: optimizeImage
};