var _environment_ = require("./environment");
var _date_ = require("./date");
var urljoin = require('url-join');
var slug = require("slug");
var _ = require("underscore");
var moment = require("moment");

function getCdnUrl (options) {

    var result = null;

    if (options && options.baseUrl) {

        result = options.baseUrl;
    } else if (_environment_.hasVariable("CDN_URL")) {

        result = _environment_.getVariable("CDN_URL");
    } else {

        //This shouldnt be here, always set CDN_URL;
        return "https://cdn.otuzbeslik.com";
    }

    return result;
}

function setUrlParam (url, queryVars) {

    var firstSeperator = (url.indexOf('&') !=-1 ? '&' : '?');
    var queryStringParts = new Array();

    for(var key in queryVars) {
        queryStringParts.push(key + '=' + queryVars[key]);
    }

    var queryString = queryStringParts.join('&');
    return url + firstSeperator + queryString;
}

function getAssetUrl (url, options) {

    if (_.isEmpty(url)) {
        
        return null;
    }

    var result = url;
    options = options || {};

    if(!isAbsoluteUrl(url)) {

        url = url.trim();

        if(_environment_.isProduction() || options.isProduction) {

            var cdnUrl = getCdnUrl(options);

            result = urljoin(cdnUrl, url);
        } else {

            if(!url.startsWith('/')) {

                result = '/' + url;
            }

            result = result.replace(/\/+/g, '\/');
            result = setUrlParam(result, {
                "v": moment().unix()
            })
        } 
    }

    return result;
}

function isAbsoluteUrl (url) {

    var isAbsolute = /^https?:\/\//i;
    return isAbsolute.test(url);
}

function toSlug (text) {

    return text ? slug(text).toLowerCase() : null
}

module.exports = {

    setUrlParam: setUrlParam,
    toSlug: toSlug,
    getCdnUrl: getCdnUrl,
    getAssetUrl: getAssetUrl,
    isAbsoluteUrl: isAbsoluteUrl,
    ob: {
        generateForAnnouncement: function (record) {

            if(record && 
                record.title) {

                var title = toSlug(record.title);
                const createdDate = toSlug(_date_.now());

                return "/duyuru/" + createdDate + "/" + title;
            }
        },
        generateForLocation: function (record) {

            if(record && record.name) {

                var _name = toSlug(record.name);

                return "/mekan/" + _name;
            }
        },

        generateForMerchandise: function (record) {

            if (record 
                && record.name
                && record.ownerName) {

                var _name = toSlug(record.name);
                var _ownerName = toSlug(record.ownerName);

                return "/urun/" + _ownerName + "/" + _name;
            }
        },

        generateForGallery: function (record) {

            if (record 
                && record.name
                && record.ownerName) {

                var _name = toSlug(record.name);
                var _ownerName = toSlug(record.ownerName);

                return "/galeri/" + _ownerName + "/" +_name;
            }
        },

        generateForGalleryItem: function (record) {

            if (record 
                && record.galleryName 
                && record.name
                && record.ownerName) {

                var _name = toSlug(record.name);
                var _galleryName = toSlug(record.galleryName);
                var _ownerName = toSlug(record.ownerName);

                return "/galeri/" + _ownerName + "/" +_name + "/" + _galleryName;
            }
        },

        generateForEvent: function (record) {

            if(record 
                && record.summary
                && record.startDate) {

                var _summary = toSlug(record.summary);
                var _startDate = _date_.dbDateToFormat(record.startDate, _date_.dateFormats.date)
                _startDate = toSlug(_startDate);

                var _locationName = toSlug(record.locationName);
                var _ownerName = toSlug(record.ownerName);

                return "/etkinlikler" 
                    + "/" + (_ownerName || _locationName)
                    + "/" + _startDate 
                    + "/" + _summary;
            }
        },

        generateForOpportunity: function (record) {

            if (record
                && record.title
                && record.startDate
                && record.endDate
                && record.ownerName) {

                var _title = toSlug(record.title);

                var _startDate = _date_.dbDateToFormat(record.startDate, _date_.dateFormats.date)
                _startDate = toSlug(_startDate);

                var _endDate = _date_.dbDateToFormat(record.endDate, _date_.dateFormats.date)
                _endDate = toSlug(_endDate);
                var _ownerName = toSlug(record.ownerName);

                return "/firsatlar"
                    + "/" + _ownerName
                    + "/" + _title
                    + "/" + _startDate
                    + "-" + _endDate;
            }
        },

        generateForPost: function (record) {

            if (record && record.title) {

                var _title = toSlug(record.title);

                return "/yazilar/" + _title;
            }
        },

        generateForArtist: function (record) {

            if (record && record.name) {

                var _name = toSlug(record.name);

                return "/kisi/" + _name;
            }
        },

        generateForGroup: function (record) {

            if (record && record.name) {

                var _name = toSlug(record.name);

                return "/grup/" + _name;
            }
        },

        generateForWorkshopSession: function (record) {

            if (record
                && record.title
                && record.startDate
                && record.endDate
                && record.locationName) {

                var _title = toSlug(record.title);
                var _startDate = _date_.dbDateToFormat(record.startDate, _date_.dateFormats.date)
                var _endDate = _date_.dbDateToFormat(record.endDate, _date_.dateFormats.date)

                _startDate = toSlug(_startDate);
                _endDate = toSlug(_endDate);

                var _locationName = toSlug(record.locationName);

                return "/atolyeler"
                    + "/" + _locationName
                    + "/" + _startDate + "-" + _endDate
                    + "/" + _title;
            }
        }
    }
};