var _array_ = require("../../src/array");
var _environment = require("../../src/environment");
var _ = require("underscore");
var chai = require('chai');
var expect  = chai.expect;

describe("Array", function() {

    before(function(done) {

        _environment.setAsTest();
        done();
    });

    after(function(done) {

        _environment.setAsTest();
        done();
    });

    it('toString: can return null when a null array is passed', function(done) {
    
        _environment.setAsTest();

        expect(_array_.toString(null)).to.be.null;
        done();
    });

    it('toString: can return empty string when an empty array is passed', function(done) {
    
        _environment.setAsTest();

        expect(_array_.toString([])).to.equal("");
        done();
    }); 

    it('toString: can return string when an array is passed', function(done) {
    
        _environment.setAsTest();

        expect(_array_.toString([1, 2, 3])).to.equal("1,2,3");
        done();
    });

    it('toString: can change delimiter for', function(done) {
    
        _environment.setAsTest();

        expect(_array_.toString([1, 2, 3], '-')).to.equal("1-2-3");
        done();
    });

    it('getArrayWithValue can get array with value', function(done) {
        
        _environment.setAsTest();

        var testData = {
            emptyString: '',
            emptyArrayString: '[]',
            nullValue: null,
            emptyArray: [],
            normalArray: [1, 2, 3],
            arrayStringInArrayFormat: "[1, 2, 3]",
            arrayStringInArrayFormatWithDelimiter: "1,2,3",
            arrayStringInArrayFormatWithDifferentDelimiter: "1-2-3",
            arrayStringWithValuesAndRemoveText: " LOCATION_1, GROUP_2, ARTIST_3_SMT",
            arrayStringWithValueAndRemoveText: "LOCATION_21862875",
            arrayStringWithValues: "1, 2, 3",
            onlyString: "19",
            onlyInt: 19
        };

        expect(_array_.getArrayWithValue(testData.emptyString)).to.eql(_array_.emptyArray);
        expect(_array_.getArrayWithValue(testData.emptyArrayString)).to.eql(_array_.emptyArray);
        expect(_array_.getArrayWithValue(testData.nullValue)).to.eql(_array_.emptyArray);
        expect(_array_.getArrayWithValue(testData.emptyArray)).to.eql(_array_.emptyArray);
        expect(_array_.getArrayWithValue(testData.normalArray)).to.eql([1, 2, 3]);
        expect(_array_.getArrayWithValue(testData.arrayStringInArrayFormat)).to.eql([1, 2, 3]);
        expect(_array_.getArrayWithValue(testData.arrayStringInArrayFormatWithDelimiter)).to.eql([1, 2, 3]);
        expect(_array_.getArrayWithValue(testData.arrayStringInArrayFormatWithDifferentDelimiter, { delimiter: '-' })).to.eql([1, 2, 3]);
        expect(_array_.getArrayWithValue(testData.arrayStringWithValuesAndRemoveText, { removeText: true })).to.eql([1, 2, 3]);
        expect(_array_.getArrayWithValue(testData.arrayStringWithValueAndRemoveText, { removeText: true })).to.eql([21862875]);
        expect(_array_.getArrayWithValue(testData.arrayStringWithValuesAndRemoveText)).to.eql(["LOCATION_1", "GROUP_2", "ARTIST_3_SMT"]);
        expect(_array_.getArrayWithValue(testData.arrayStringWithValueAndRemoveText)).to.eql(["LOCATION_21862875"]);
        expect(_array_.getArrayWithValue(testData.arrayStringWithValueAndRemoveText, { removeText: true })).to.eql([21862875]);
        expect(_array_.getArrayWithValue(testData.arrayStringWithValues)).to.eql([1, 2, 3]);
        expect(_array_.getArrayWithValue(testData.onlyString)).to.eql([19]);
        expect(_array_.getArrayWithValue(testData.onlyInt)).to.eql([19]);

        done();
    });
    
    it('can prepare for autocomplete', function(done) {
    
        _environment.setAsTest();

        var testData = [
            {
                id: 1,
                name: "Test 1",
                age: 22
            },
            {
                id: 2,
                name: "Test 3",
                age: 23
            }
        ];

        var prepared = _array_.prepareForAutocomplete(testData, 'name');
        var preparedObj = JSON.parse(prepared);

        expect(typeof(prepared)).to.equal("string");
        expect(preparedObj[0].id).to.equal(1);
        expect(preparedObj[1].id).to.equal(2);
        expect(preparedObj[0].text).to.equal("Test 1");
        expect(preparedObj[1].text).to.equal("Test 3");

        done();
    });

    it('can return null for null autocomplete input', function(done) {
    
        _environment.setAsTest();

        var testData = null;

        var prepared = _array_.prepareForAutocomplete(testData, 'name');
        expect(prepared).to.be.null;
        
        done();
    });
    
    it('can handle null input for pluck many', function(done) {
    
        _environment.setAsTest();

        var testData = null;
        
        var empty1 = _array_.pluckMany(testData, ["id", "name"]);
        var empty2 = _array_.pluckMany(testData, null);

        expect(empty1).to.be.undefined;
        expect(empty2).to.be.undefined;

        done();
    });

    it('can pluck many', function(done) {
    
        _environment.setAsTest();

        var plucked = _array_.pluckMany([
            {
                id: 1,
                name: "Test 1",
                age: 22
            },
            {
                id: 2,
                name: "Test 3",
                age: 23
            }
        ], ["id", "name"]);

        expect(plucked).to.have.lengthOf(2);
        expect(Object.keys(plucked[0])).to.have.lengthOf(2);
        expect(Object.keys(plucked[1])).to.have.lengthOf(2);
        expect(plucked[0].id).to.equal(1);
        expect(plucked[1].id).to.equal(2);
        expect(plucked[0].age).to.be.undefined
        expect(plucked[1].age).to.be.undefined;

        done();
    });

    it('can process func partially', function(done) {
    
        _environment.setAsTest();

        var largeArray = [2, 1, 2, 4, 5, 7, 8, 8];

        _array_.processPartial(largeArray, 2, function (partial) {

            return new Promise(function (resolve, reject) {

                try {

                    var sum = _.reduce(partial, function(memo, num){ return memo + num; }, 0);
                    resolve(sum);
                } catch (e) {

                    reject(e);
                }
            });
        }).then(function (results) {

            expect(results).not.to.be.null;
            expect(results.length).to.eql(4);
            expect(results[0]).to.eql(3);
            expect(results[1]).to.eql(6);
            expect(results[2]).to.eql(12);
            expect(results[3]).to.eql(16);
            done();
        });
    });
    
    it('can process func partially with max byte size', function(done) {
    
        _environment.setAsTest();

        var largeArray = [2, 1, 2, 4, 5, 7, 8, 8];

        _array_.processPartial(largeArray, {
            maxBytes: 20
        }, function (partial) {

            return new Promise(function (resolve, reject) {

                try {

                    var sum = _.reduce(partial, function(memo, num){ return memo + num; }, 0);
                    resolve(sum);
                } catch (e) {

                    reject(e);
                }
            });
        }).then(function (results) {

            expect(results).not.to.be.null;
            expect(results.length).to.eql(4);
            expect(results[0]).to.eql(3);
            expect(results[1]).to.eql(6);
            expect(results[2]).to.eql(12);
            expect(results[3]).to.eql(16);
            done();
        });
    });
});
    