var _url = require("../../src/url");
var _environment = require("../../src/environment");

var chai = require('chai');
chai.use(require('chai-string'));
var expect  = chai.expect;

describe("Url", function() {

    before(function(done) {

        done();
    });

    beforeEach(function(done) {

        _environment.setAsTest();
        done();
    });

    after(function(done) {

        _environment.setAsTest();
        done();
    });

    it('can get panel dev url', function(done) {
    
        _environment.setAsDevelopment();

        expect(_url.getAssetUrl("img/test.png")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl(" img/test.png")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl("img/test.png ")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl("//img/test.png")).to.startsWith("/img/test.png");
        done();
    });

    it('can get prod url for panel', function(done) {

        _environment.setAsProduction();

        _environment.setVariable("CDN_URL", "https://d22e6md4l9gwjr.cloudfront.net");

        expect(_url.getAssetUrl("img/test.png")).to.equal("https://d22e6md4l9gwjr.cloudfront.net/img/test.png");
        expect(_url.getAssetUrl(" img/test.png")).to.equal("https://d22e6md4l9gwjr.cloudfront.net/img/test.png");
        expect(_url.getAssetUrl("img/test.png ")).to.equal("https://d22e6md4l9gwjr.cloudfront.net/img/test.png");
        expect(_url.getAssetUrl("//img/test.png")).to.equal("https://d22e6md4l9gwjr.cloudfront.net/img/test.png");
        done();
    });

    it('can setBaseUrl', function(done) {

        _environment.setAsProduction();

        var options = {
            baseUrl: "https://d22e6md4l9gwjr.cloudfront.net"
        };

        expect(_url.getAssetUrl("img/test.png", options)).to.equal("https://d22e6md4l9gwjr.cloudfront.net/img/test.png");
        expect(_url.getAssetUrl(" img/test.png", options)).to.equal("https://d22e6md4l9gwjr.cloudfront.net/img/test.png");
        expect(_url.getAssetUrl("img/test.png ", options)).to.equal("https://d22e6md4l9gwjr.cloudfront.net/img/test.png");
        expect(_url.getAssetUrl("//img/test.png", options)).to.equal("https://d22e6md4l9gwjr.cloudfront.net/img/test.png");
        done();
    });

    it('can get dev url', function(done) {
    
        _environment.setAsDevelopment();

        expect(_url.getAssetUrl("img/test.png")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl(" img/test.png")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl("img/test.png ")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl("//img/test.png")).to.startsWith("/img/test.png");
        done();
    });

    it('can get prod url', function(done) {

        _environment.setAsProduction();

        _environment.clearVariable("CDN_URL");

        expect(_url.getAssetUrl("img/test.png")).to.equal("https://cdn.otuzbeslik.com/img/test.png");
        expect(_url.getAssetUrl(" img/test.png")).to.equal("https://cdn.otuzbeslik.com/img/test.png");
        expect(_url.getAssetUrl("img/test.png ")).to.equal("https://cdn.otuzbeslik.com/img/test.png");
        expect(_url.getAssetUrl("//img/test.png")).to.equal("https://cdn.otuzbeslik.com/img/test.png");
        done();
    });

    it('can get test url', function(done) {

        expect(_url.getAssetUrl("img/test.png")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl(" img/test.png")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl("img/test.png ")).to.startsWith("/img/test.png");
        expect(_url.getAssetUrl("//img/test.png")).to.startsWith("/img/test.png");
        
        done();
    });

    it('can handle empty url', function(done) {
    
        expect(_url.getAssetUrl("")).to.be.null;
        expect(_url.getAssetUrl(null)).to.be.null;
        expect(_url.getAssetUrl(undefined)).to.be.null;
        
        done();
    });

    it('can detect absolute urls', function(done) {
     
        expect(_url.isAbsoluteUrl("/img/test.png")).to.be.false;
        expect(_url.isAbsoluteUrl("http://www.otuzbeslik.com")).to.be.true;

        done();
    });   
    
    it('can change CDN url from environment variables', function(done) {
     
        _environment.setAsProduction();

        _environment.clearVariable("CDN_URL");

        var relativePath = "img/test.png";
        var oldUrl = _url.getAssetUrl(relativePath);
        expect(oldUrl).to.equal("https://cdn.otuzbeslik.com/img/test.png");

        _environment.setVariable("CDN_URL", "somethingcdn.otuzbeslik.com");
        _url = require("../../src/url");

        var newUrl = _url.getAssetUrl(relativePath);
        expect(newUrl).to.equal("somethingcdn.otuzbeslik.com/img/test.png");
        expect(newUrl).not.to.equal(oldUrl);

        done();
    }); 
    
    it('can generate a url for location', function(done) {

        var record = {
            name: "Sir Winston Tea 88's türkçe_with@underscore"
        };

        var url = _url.ob.generateForLocation(record);
      
        expect(url).to.equal("/mekan/sir-winston-tea-88s-turkce_withunderscore");

        done();
    });  

    it('can generate a url for event', function(done) {

        var record = {
            summary: "Sir Winston Tea 88's türkçe_with@underscore",
            startDate: "2016-10-07 17:30:00",
            locationName: "Sir Winston Tea"
        };

        var url = _url.ob.generateForEvent(record);
      
        expect(url).to.equal("/etkinlikler/sir-winston-tea/2016-10-07/sir-winston-tea-88s-turkce_withunderscore");

        done();
    });  

    it('can generate a url for opportunity', function (done) {

        var record = {
            title: "İnanılmaz Enteresan Türkçe Başlık",
            startDate: "2020-10-10 10:20:00",
            endDate: "2020-10-10 10:21:00",
            ownerName: "bilsay"
        };

        var url = _url.ob.generateForOpportunity(record);

        expect(url).to.equal("/firsatlar/bilsay/inanilmaz-enteresan-turkce-baslik/2020-10-10-2020-10-10");

        done();
    });  

    it('can generate a url for gallery', function (done) {

        var record = {
            name: "İnanılmaz Enteresan Türkçe Başlık",
            ownerName: "bilsay"
        };

        var url = _url.ob.generateForGallery(record);

        expect(url).to.equal("/galeri/bilsay/inanilmaz-enteresan-turkce-baslik");

        done();
    });  
    it('can generate a url for post', function (done) {

        var record = {
            title: "İnanılmaz Enteresan Türkçe Başlık"
        };

        var url = _url.ob.generateForPost(record);

        expect(url).to.equal("/yazilar/inanilmaz-enteresan-turkce-baslik");

        done();
    });  

    it('can generate a url for artist', function (done) {

        var record = {
            name: "Osman Çiğer"
        };

        var url = _url.ob.generateForArtist(record);

        expect(url).to.equal("/kisi/osman-ciger");

        done();
    });

    it('can generate a url for group', function (done) {

        var record = {
            name: "Başarı Derneği"
        };

        var url = _url.ob.generateForGroup(record);
        expect(url).to.equal("/grup/basari-dernegi");

        done();
    }); 

    it('can add url param', function(done) {

        expect(_url.setUrlParam("https://www.otuzbeslik.com/a=1&b=32", { "v": 1234 })).to.be.eql("https://www.otuzbeslik.com/a=1&b=32&v=1234");
        done();
    });
});
    